#ifndef SCOPE_QUERY_H_
#define SCOPE_QUERY_H_

#include <api/client.h>

#include <unity/scopes/SearchQueryBase.h>
#include <unity/scopes/ReplyProxyFwd.h>
#include <unity/scopes/utility/BufferedResultForwarder.h>

namespace scope {

/**
 * Represents an individual query.
 *
 * A new Query object will be constructed for each query. It is
 * given query information, metadata about the search, and
 * some scope-specific configuration.
 */
class Query: public unity::scopes::SearchQueryBase {
public:
    struct CategoryBuffer {
        unity::scopes::Category::SCPtr category;
        unity::scopes::utility::BufferedResultForwarder::SPtr buffer;
    };

    Query(const unity::scopes::CannedQuery &query,
          const unity::scopes::SearchMetadata &metadata);

    ~Query() = default;

    void cancelled() override;

    void run(const unity::scopes::SearchReplyProxy &reply) override;

private:
    void internationalVersion(const unity::scopes::SearchReplyProxy &reply,
                              const unity::scopes::CannedQuery &query);
    void germanVersion(const unity::scopes::SearchReplyProxy &reply,
                       const unity::scopes::CannedQuery &query);
    bool pushResult(const unity::scopes::SearchReplyProxy &reply,
                    unity::scopes::Category::SCPtr category,
                    const api::Client::NewsItem &news);

    api::Client client_;
};

}

#endif // SCOPE_QUERY_H_


